module MonadState where

import Exception

type St a s = s -> E (a,s)

-- Basic Combinators --

unitSt :: a -> St a s
unitSt a = \s -> unitE (a,s)

bindSt :: St a s -> (a -> St b s) -> St b s
m `bindSt` k = \s -> m s `bindE` (\(a,t) -> k a t)

errorSt :: String -> St a s
errorSt str = \s -> errorE str
{-
escapeSt :: St a s -> St a s -> St a s
f `escapeSt` g = \s -> case (f s) of
                           Left _ -> g s
                           Right a -> Right a
-}
-- general functions --


stripSt :: St a s -> s -> E a
stripSt m s = m s `bindE` (unitE . fst)

getSt :: St s s
getSt = \s -> unitE (s,s)

putSt :: s -> St () s
putSt s = \_ -> unitE ((),s)

liftEtoSt :: E a -> St a s
liftEtoSt e = \s -> e `bindE` (\a -> unitE (a,s))

liftEfunSt :: (s -> E a) -> St a s
liftEfunSt f = \s -> f s `bindE` (\a -> unitE (a,s))

liftFunSt :: (s -> a) -> St a s
liftFunSt f = \s -> unitE (f s,s)

appendSt :: St [a] s -> St [a] s -> St [a] s
appendSt m k = m `bindSt` (\as ->
               k `bindSt` (\as1 -> unitSt (as++as1)))

concatSt :: [St [a] s] -> St [a] s
concatSt = foldr appendSt (unitSt [])

zipSt :: [a] -> [b] -> St [(a,b)] s
zipSt as bs = liftEtoSt (zipE as bs)

updateSt :: (s -> s) -> St () s
updateSt f = getSt `bindSt` (putSt . f)

skipSt :: St () s
skipSt = updateSt id

zipWithSt :: (a -> b -> St c s) -> [a] -> [b] -> St [c] s
zipWithSt z (a:as) (b:bs) = z a b `consSt` zipWithSt z as bs
zipWithSt _     _      _  = unitSt []

consSt :: St a s -> St [a] s -> St [a] s
consSt m k = m `bindSt` (\a ->
             k `bindSt` (\as -> unitSt (a:as)))

sequenceSt :: [St a s] -> St [a] s
sequenceSt [] = unitSt []
sequenceSt (k:ks) = k `consSt` sequenceSt ks 

mapSt :: (a -> St b s) -> [a] -> St [b] s
mapSt f [] = unitSt []
mapSt f (a:as) = f a `consSt` mapSt f as
