module NewPPrint(printObj, printMeth, printDef, printEnv)

where

import Pretty
import PrettyPrint
import AbsSynt
import Env

pparen False x = x
pparen True  x = parens x 

dot = char '.'
equal = char '='
lparp = char '('
rparp = char ')'
brackets p = char '[' <> p <> char ']'
assig = text "<-"
arrow = text "->"
assigf = text ":="
ifp = text "if"
thenp = text "then"
elsep = text "else"
space = text " "
truep = text "True"
falsep = text "False"

ppiSeqSepAbove :: String -> (a -> Doc) -> [a] -> Doc
ppiSeqSepAbove _ _ [] = empty
ppiSeqSepAbove sep pf xs = foldr1 (\a b -> (a <> text sep) $$$ b) $ map pf xs

--******* Instances for objects *******--

{- The class PPrint is used for taking care of parentheses nesting -}

class PPrint a where
 pPrint :: Int -> a -> Doc

instance PPrint Method where
 
   pPrint p (Met (n,m)) = 
            pparen (p > 0) (lparp <> text n <> rparp <> pPrint 0 m)

instance PPrint SigTerm where

   pPrint p (Var i) = text i
   pPrint p (Obj oflds) =
            pparen (p > 0) (brackets  toflds)
            where toflds = ppiSeqSepAbove "," (pPrint 0 . PrOFld) oflds
   pPrint p (Sel o n) = pparen (p > 0) (pPrint 0 o <> dot <> text n)
   pPrint p (UpdM o l m) = 
      pparen (p > 0) ((pPrint 0 o <> dot <> text l) $$ (assig <> pPrint 0 m))  

   pPrint p (App e1 e2) = 
        pparen (p>9) (pPrint 0 e1 <+> pPrint 10 e2 )
   pPrint p (Lam n e) = 
        pparen (p>0) (text "\\" <> text n <+> arrow <+> pPrint 0 e)
   pPrint p (Num n) = text (show n)
   pPrint p TT = truep
   pPrint p FF = falsep
   pPrint p (Binop op e1 e2) = 
      pparen (p > 0) (pPrint 0 e1 <> space <> text op <> space <> pPrint 0 e2)
   pPrint p (SIf b e1 e2) =
      pparen (p > 0) ((ifp <> pPrint 0 b) $$ 
                      (thenp <> pPrint 0 e1) $$ (elsep <> pPrint 0 e2))
   pPrint _ _ = error "pPrint:not defined" 

data PrintOFld = PrOFld (Label,Method) deriving (Show)

instance PPrint PrintOFld where
  pPrint p (PrOFld (n,o)) = text n <+> equal <+> pPrint 0 o

-- Instances for Enviroment and definition(s)

instance PPrint Def where
   pPrint p (Def (n,e)) = (text n <+> equal <+> pPrint 0 e)

instance PPrint Env where
   pPrint _ (Env dfs) = ppiSeqSepAbove ";" (pPrint 0) (reverse dfs)

{- Instances for class Printable -}

instance Printable Method where
  ppi = pPrint 0

instance Printable SigTerm where
  ppi = pPrint 0

instance Printable PrintOFld where
  ppi = pPrint 0

instance Printable Def where
  ppi = pPrint 0

instance Printable Env where
  ppi = pPrint 0

-- Pretty printing functions, these are the exported ones

printObj  :: SigTerm -> String
printObj = pp

printMeth :: Method -> String
printMeth = pp

printDef :: Def -> String
printDef = pp

printEnv :: Env -> String
printEnv = pp

