module Reduct where

import Exception
import AbsSynt
import Lex
import Env
import MonadSTE
import State
import Subst

compile :: SigTerm -> StS SigTerm
compile sgt =
   case sgt of
     (App e1 e2) -> 
        do p <- compile e1
           q <- compile e2
           return (Sel (UpdM p "arg" (Met ("_",q))) "val")

     (Lam x e) -> 
        do p <- compile e
           let selarg = Sel (Var x) "arg"
           psubst <- subst p x selarg
           return (Obj [("arg",Met (x,selarg)),("val",Met (x,psubst))])

     _ -> return sgt

reduct :: SigTerm -> StS SigTerm
reduct sgt = 
 do csgt <- compile  sgt
    case csgt of
      (Var x) -> 
         do o <- lookupEnvStS x
            reduct o

      (Obj lms) -> return csgt

      (Sel obj l) -> 
         do o <- reduct obj
	    case o of
	      (Obj lms) ->
	        do m <- lookUpMethodAss l lms
		   case m of
		      Met (x,obj) -> 
		        do obj' <- subst  obj x o
                           reduct obj'
		      _  -> errorSTE "reduct: m is not abstraction"
	      _  -> errorSTE "reduct,Sel: o is not an object"

      (UpdM a l m) ->
         do o <- reduct a
	    case o of
	      (Obj lms) -> liftEtoSTE (do assg <- updateLabelAss l m lms
                                          return (Obj assg))
	        
	      _         -> errorSTE "reduct,UpdM: o is not an object"
     
      (Num n) -> return (Num n)

      TT -> return TT

      FF -> return FF

      (Binop op e1 e2) -> 
          do ce1 <- reduct e1
             ce2 <- reduct e2
             return (if op `elem` relops then calcrelop op ce1 ce2
                     else if op `elem` boolops then calcboolop op ce1 ce2
                          else arithm op ce1 ce2)


      (SIf b t e) ->
          do cb <- reduct b
             case cb of
                TT -> reduct t 
                FF -> reduct e
                _ -> errorSTE "reduc,Sif: not a boolean!!" 
                                        
      _ -> errorSTE "reduct: csgt is not an object"

reductProg :: Program -> StS SigTerm
reductProg (Program (Env dfs,o)) =
      do envact <- getEnvStS 
         putEnvStS (addDefsEnv dfs envact)
         reduct o


arithm op (Num m) (Num n) =
   Num     (case op of
            "+" -> m + n
            "-" -> m - n
            "*" -> m * n
	    "/" -> m `div` n)
arithm _ _ _ = error "arithm, no son numeros"

calcrelop op (Num m) (Num n) =
   case op of
      "==" -> if m == n then TT else FF
      "/=" -> if m /= n then TT else FF
      "<=" -> if m <= n then TT else FF
      ">=" -> if m >= n then TT else FF
      ">" -> if m > n then TT else FF
      "<" -> if m < n then TT else FF
calcrelops _ _ _ = error "calcrelops, no son numeros"

calcboolop op b1 b2 =
   case op of
      "||" -> if b1 == TT then TT else b2
      "&&" -> if b1 == FF then FF else b2
      _ -> error "calcboolop, not defined operator"
