module Subst where

import AbsSynt
import MonadSTE
import State



makeFresh :: StS Name
makeFresh = do {v <- getFreshStS; putFreshStS (v+1);return ("_" ++ show v)}

substMethod :: Method -> Name -> SigTerm -> StS Method
substMethod (Met (y,b)) x c = 
       do  y'  <- makeFresh
           b'  <-  subst b y (Var y')
           b'' <-  subst b' x c   
           return (Met (y',b''))

class Subst a where
   subst :: a -> Name -> a -> StS a 

instance Subst SigTerm where
   subst (Var z) x c = return (if z == x then c else (Var z))
   subst (Obj lms) x c = 
     let labels = map fst lms
     in  do lms' <- sequenceSTE (map (\(_,m) -> substMethod m x c) lms)
            return (Obj (zip labels lms'))
   subst (Sel a l) x c = 
       do a' <- subst a x c
          return (Sel a' l)
   subst (UpdM a l m) x c = 
       do a' <- subst a x c  
          m' <- substMethod m x c
          return (UpdM a' l m')
   subst (App e1 e2) x c = 
       do p <- subst e1 x c
          q <- subst e2 x c
          return (App p q)
   subst (Lam y e) x c =
       do  y'  <- makeFresh
           e'  <-  subst e y (Var y')
           e'' <-  subst e' x c   
           return (Lam y' e'')
   subst (Num n) x c = return (Num n)
   subst (TT) x c = return (TT)
   subst (FF) x c = return (FF)
   subst (Binop op e1 e2) x c =
         do p <- subst e1 x c
            q <- subst e2 x c
            return (Binop op p q)
   subst (SIf b e1 e2) x c = 
          do cb <- subst b x c
	     p <- subst e1 x c
             q <- subst e2 x c
             return (SIf cb p q)
   subst _ _ _ = errorSTE "subst: not yet defined"
