
(defvar fob-mode-map ()
  "local keymap for fob mode buffers")

(defconst fob-prog-name "fob" 
  "*Name of program to type check fob definitions.")


(defconst fob-process-name "fob" "*The name of the fob-process")




(if fob-mode-map
    ()
  (setq fob-mode-map (make-sparse-keymap))
 ;; (define-key fob-mode-map "\r"  'fob-electric-return)
 ;; (define-key fob-mode-map "\t" 'fob-indent-line)
  (define-key fob-mode-map "\C-c\C-s" 'fob-pop-to-shell)
  (define-key fob-mode-map "\C-c\C-u" 'fob-save-buffer-use-file)
  (define-key fob-mode-map "\C-c\C-a" 'fob-add-definition)
;;  (define-key fob-mode-map "\C-c\C-d" 'fob-delete-definition)

  (define-key fob-mode-map "\C-c\C-g" 'fob-send-region)
  (define-key fob-mode-map "\C-c\C-f" 'fob-run-on-file)
;;  (define-key fob-mode-map "\C-c\C-l" 'fob-run-on-files)
  (define-key fob-mode-map "\C-c\C-e" 'fob-show-environment)
  (define-key fob-mode-map "\C-c\C-c" 'fob-get-definition)
;;  (define-key fob-mode-map "\C-c\C-t" 'fob-get-type)
  (define-key fob-mode-map "\C-c\C-w" 'fob-eval-whnf)
  (define-key fob-mode-map "\C-c\C-m" 'fob-compile)
  (define-key fob-mode-map "\C-c\C-p" 'fob-parse)
  (define-key fob-mode-map "\C-c\C-h" 'fob-help)
  (define-key fob-mode-map "\C-c\C-x" 'fob-clear-definitions)
  (define-key fob-mode-map "\C-c\C-q" 'fob-quit-system)
  (define-key fob-mode-map "\C-c\C-r" 'fob-reload-buffer))


(defun fob-set-syntax-table ()
  "Set the syntax table for fob-mode."
  (setq fob-mode-syntax-table (make-syntax-table))
  (set-syntax-table fob-mode-syntax-table)
  (modify-syntax-entry ?\' "w")
  (modify-syntax-entry ?_  "w")
  (modify-syntax-entry ?:  "_")
  (modify-syntax-entry ?%  "_ 14")
  (modify-syntax-entry ?- "_ 23")
;;  (modify-syntax-entry ?<  "(>")
;;  (modify-syntax-entry ?>  ")<")
  (set-syntax-table fob-mode-syntax-table))


;; font-lock

(defconst  fob-font-lock-keywords (purecopy
  (list
   '("^[ \t]*\\(let\\|in\\|use\\)\\>"
     1 font-lock-doc-string-face)
   '("\\<\\(Type\\|Set\\)\\>"
     1 font-lock-type-face)
   '("^\\(\\sw+\\)[ \t]*[(:]"
     1  font-lock-function-name-face t t )
   '("^\\s-*\\(\\sw+\\)\\s-*:" 1 font-lock-variable-name-face)))
  "Additional expressions to highlight in fob mode.")

(defun fob-mode ()
  "Fob mode.
Major mode to edit definitions for Fob evaluator.

Commands:
\\{fob-mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq mode-popup-menu fob-menu)
  (set-buffer-menubar current-menubar)
  (add-submenu nil fob-menu)
  (fob-set-syntax-table)
  (setq major-mode 'fob-mode)
  (setq mode-name "Fob Definitions")
  (make-local-variable 'fob-shell-visible)
  (setq fob-shell-visible nil) ;; inferior shell no visible
  (use-local-map fob-mode-map))


(defconst fob-menu
  '( "Fob"
     "---"
    ["Init Fob evaluator"		fob-pop-to-shell            	t]
    ["Load this file"                  fob-save-buffer-use-file     t]
    ["Load this definition"            fob-add-definition           t]
    ["Load region"                     fob-send-region              t]
    ["Reload this file"                 fob-reload-buffer            t]
    ["Load file ..."	        	fob-run-on-file	       	t]
  ;;  ["Load file of files..."	        fob-run-on-files	       	t]
    ["Show constant"     	        fob-get-definition	       	t]
  ;;  ["Type of constant"	                fob-get-type 	       	t]
    ["Evaluate"	                        fob-eval-whnf 	       	t]
    ["Compile"	                        fob-compile 	       	t]
    ["Parse"	                        fob-parse 	       	t]
    ["Show environment"	        	fob-show-environment	       	t]
  ;;  ["Delete last definition"           fob-delete-definition        t]
    ["Reset environment"                fob-clear-definitions        t]
    ["Help "                            fob-help                     t]
    ["Quit "                            fob-quit-system              t]
    "---"
    ))


;;
;; Inferior shell
;;

(defvar fob-shell-map nil "The mode map for fob-shell.")

(defun fob-shell ()
  "Inferior shell invoking fob.
It is not possible to have more than one shell running fob.
Like the shell mode with the additional command:

\\[fob-run-on-file]\t Runs haskell on the file.
\\{fob-shell-map}

Variables controlling the mode:

fob-prog-name (default \"fob\")
    The string used to invoke the haskell program.



fob-process-name (default \"fob\")
    The name of the process running haskell.

fob-shell-prompt-pattern (default \"^> *\")
    The prompt pattern.

Runs fob-shell-hook if not nil."
  (interactive)
  (if (not (process-status fob-process-name))
      (save-excursion			; Process is not running
	(message "Starting fob...")	; start up a new process
	(require 'shell)
	(set-buffer 
	      (make-comint fob-process-name fob-prog-name))
	(erase-buffer) ; Erase the buffer if a previous
	(if fob-shell-map		; process died in there
	    ()
	  (setq fob-shell-map (copy-keymap shell-mode-map))
	  (define-key fob-shell-map "\C-c\C-f" 'haskell-run-on-file))
;;	  (define-key fob-shell-map "\C-c\C-f" 'fob-run-on-file))
	(use-local-map fob-shell-map)
	(make-local-variable 'shell-prompt-pattern)
	(setq shell-prompt-pattern ">")
	(setq major-mode 'fob-shell)
	(setq mode-name "FOB Shell")
	(set-process-filter (get-process fob-process-name)
			    'fob-process-filter)
	(message "Starting Fob...done.")
	(run-hooks 'fob-shell-hook))))

(defun fob-process-filter (proc str)
  (let ((cur (current-buffer))
	(pop-up-windows t))
    (pop-to-buffer (concat "*" fob-process-name "*"))
    (goto-char (point-max))
    (if (string= str "\b\b\b  \b\b\b")
	(backward-delete-char 4)
      (insert str))
 ;; Estudio si hubo problemas
 ;;   (if (string-match "\\(^Problem:.*\\)$" str)
 ;;	(progn
 ;;	  (message (match-string 1 str))
 ;;	  (beep)))
    (set-marker (process-mark proc) (point-max))
    (pop-to-buffer cur)))
  
(defun fob-pop-to-shell ()
  (interactive)
  (fob-shell)
  (pop-to-buffer (concat "*" fob-process-name "*")))

(defun fob-run-on-file (fil)
  (interactive "FLoad file : ")
  (fob-shell)
  (save-some-buffers)
  (process-send-string fob-process-name
	       (concat "load " (base-name fil)  "\n"
		     )))

;; estas las puse yo 07/08

(defun fob-run-on-files (fil)
  (interactive "FRun fob on : ")
  (fob-shell)
  (save-some-buffers)
  (process-send-string fob-process-name
	       (concat "loadf " (base-name fil)  "\n"
		     )))


(defun fob-get-definition (cname)
  (interactive "sName of constant : ")
  (fob-shell)
  (process-send-string fob-process-name
	       (concat "getdef " cname  "\n"
		     )))

(defun fob-get-type (cname)
  (interactive "sName of constant : ")
  (fob-shell)
  (process-send-string fob-process-name
	       (concat "type " cname  "\n"
		     )))

(defun fob-eval-whnf (exp)
  (interactive "sObject : ")
  (fob-shell)
  (process-send-string fob-process-name
	       (concat "eval " exp  "\n"
		     )))

(defun fob-compile (exp)
  (interactive "sObject : ")
  (fob-shell)
  (process-send-string fob-process-name
	       (concat "comp " exp  "\n"
		     )))

(defun fob-parse (exp)
  (interactive "sObject : ")
  (fob-shell)
  (process-send-string fob-process-name
	       (concat "parse " exp  "\n"
		     )))

;; 

(defun fob-save-buffer-use-file ()
  "Save the buffer, and send a `use file' to the inferior shell
running Fob."
  (interactive)
  (let (file)
    (if (setq file (buffer-file-name))	; Is the buffer associated
	(progn				; with file ?
	  (save-buffer)
	  (fob-shell)
	  (process-send-string fob-process-name
		       (concat "load " (base-name file) "\n")))
      (error "Buffer not associated with file."))))

(defvar fob-home-dir "/inco/09/usr/gustun/")

(defun base-name (file)
;;   (concat fob-home-dir (car (last (split-string file "~")))))
  (car (last(split-string file "/"))))

;; estas tambien las agregue yo

(defun fob-show-environment ()
 "shows the current environment"
  (interactive)
  (fob-shell)
  (process-send-string fob-process-name "env \n"))

(defun fob-delete-definition ()
 "shows the current environment"
  (interactive)
  (fob-shell)
  (process-send-string fob-process-name "del \n"))

(defun fob-help ()
 "shows the help of the system"
  (interactive)
  (fob-shell)
  (process-send-string fob-process-name "help \n"))

(defun fob-quit-system ()
 "quits the system"
  (interactive)
  (fob-shell)
  (process-send-string fob-process-name "quit \n"))

;;

(defun fob-clear-definitions ()
  "Borra todas las definiciones en el ambiente 
del proceso Fob en ejecucion"
  (interactive)
  (fob-shell)
  (process-send-string fob-process-name "reset \n"))

(defun fob-reload-buffer ()
  (interactive)
  (fob-clear-definitions)
  (fob-save-buffer-use-file))


" This below is an attempt of defining a function for checking regions (definitions)"
" Is a modification of the function  send-region from haskell-mode.el "

;; The template used for temporary files, created when a region is
;; send to the inferior process running fob.

(defconst fob-tmp-template "/tmp/fob.tmp."
  "*Template for the temporary file, created by fob-simulate-send-region.")

(defvar fob-check-command "load"
  "The command sent to check a region/buffer.")

 (defun fob-make-temp-name (pre)
  (concat (make-temp-name pre) ".fob"))

 (defvar fob-tmp-files-list nil
  "List of all temporary files created by haskell-simulate-send-region.
Each element in the list is a list with the format:

      (\"tmp-filename\"  buffer  start-line)")
   
 (defvar fob-simulate-send-region-called-p nil
  "Has haskell-simulate-send-region been called previously.")

 (defun fob-simulate-send-region (point1 point2)
  "Simulate send region. As send-region only can handle what ever the
system sets as the default, we have to make a temporary file.
Updates the list of temporary files (fob-tmp-files-list)."
  (let ((file (expand-file-name (fob-make-temp-name fob-tmp-template))))
    ;; Remove temporary files when we leave emacs
    (if (not fob-simulate-send-region-called-p)
	(progn
	  (add-hook 'kill-emacs-hook 'fob-remove-tmp-files)
	  (setq fob-simulate-send-region-called-p t)))
    (save-excursion
      (goto-char point1)
      (setq fob-tmp-files-list
	    (cons (list file
			(current-buffer)
			(save-excursion	; Calculate line no.
			  (beginning-of-line)
			  (1+ (count-lines 1 (point)))))
		  fob-tmp-files-list)))
    (write-region point1 point2 file nil 'dummy)
    (fob-shell)
    (message "Using temporary file: %s" file)
    (send-string
     fob-process-name
     ;; string to send: load file;
     (concat (concat fob-check-command " ")
	     file "\n"))))


(defun fob-remove-tmp-files ()
  "Remove the temporary files, created by haskell-simulate-send-region, if
they still exist. Only files recorded in haskell-tmp-files-list are removed."
  (message "Removing temporary files created by haskell-mode...")
  (while fob-tmp-files-list
    (condition-case ()
 	(delete-file (car (car fob-tmp-files-list)))
      (error ()))
    (setq fob-tmp-files-list (cdr fob-tmp-files-list)))
  (message "Removing temporary files created by haskell-mode...done."))

(defun fob-send-region ()
  "Send region."
  (interactive)
  (let (start end)
    (save-excursion
      (setq end (point))
      (exchange-point-and-mark)
      (setq start (point)))
    (fob-simulate-send-region start end)))

(defun fob-send-buffer ()
  "Send the buffer."
  (interactive)
  (fob-simulate-send-region (point-min) (point-max)))


"For checking one definition"

(defun backward-begin-def ()
"Se posiciona al comienzo de una definicion.
Se asume que una definicion empieza en el comienzo de la linea"
  (interactive)
  (search-backward-regexp "^\\sw+"))


(defun forward-end-def ()
"Se posiciona al final de una definicion"
  (interactive)
  (search-forward-regexp "^\\sw+\\|\\'" () t)
  (if (not (looking-at ";"))
       (search-backward ";"))
      )

(defun fob-add-definition ()
  "Carga la definicion donde el cursor
   esta posicionado. Si esta en el medio de 2 defs.
   o al final del buffer carga la anterior;
   -NO salva el buffer-"
  (interactive)
  (save-excursion
  (let* 
      (;;comienzo de definicion
       (beg (progn			
	      (backward-begin-def)
	      (point)))
       ;;nombre de la constante
       (name-def (progn (looking-at "\\sw+")
			(match-string 0)))
       ;; fin de la definicion 
       (fin (progn			
	      (forward-word 1)
	      (forward-end-def)
	      (point)))
       ;; texto de la definicion, se eliminan los fines de linea
       (def (replace-in-string (buffer-substring beg fin)
			       "\n" " ")))
    ;; Consulta si desea o no cargar la definicion
    ( if (y-or-n-p-maybe-dialog-box 
	  (concat "Add definition of \"" name-def "\"?"))
	(progn 
	  (message (concat "Add definition`" name-def "'...")) 
	  (fob-shell)
	  (process-send-string fob-process-name 
		       (concat "add " def ";\n"))
	  (message (concat
		    "Add definifion `" name-def "'...Done" ))))
      )))


